use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
#[derive(Clone, Copy)]
pub union VkDeviceOrHostAddressKHR {
    deviceAddress: VkDeviceAddress,
    hostAddress: *mut c_void,
}

impl VkDeviceOrHostAddressKHR {
    pub fn null() -> Self {
        VkDeviceOrHostAddressKHR {
            hostAddress: ptr::null_mut(),
        }
    }

    pub fn device_address(&self) -> VkDeviceAddress {
        unsafe { self.deviceAddress }
    }

    pub fn host_address<T>(&mut self) -> &mut T {
        unsafe { &mut *(self.hostAddress as *mut T) }
    }
}

impl<'a, T> From<&'a mut T> for VkDeviceOrHostAddressKHR {
    fn from(host_address: &'a mut T) -> Self {
        VkDeviceOrHostAddressKHR {
            hostAddress: host_address as *mut T as *mut c_void,
        }
    }
}

impl<'a, T> From<&'a mut [T]> for VkDeviceOrHostAddressKHR {
    fn from(host_address: &'a mut [T]) -> Self {
        VkDeviceOrHostAddressKHR {
            hostAddress: host_address.as_mut_ptr() as *mut c_void,
        }
    }
}

impl From<VkDeviceAddress> for VkDeviceOrHostAddressKHR {
    fn from(device_address: VkDeviceAddress) -> Self {
        VkDeviceOrHostAddressKHR {
            deviceAddress: device_address,
        }
    }
}
